<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       http://wpgenie.org
 * @since      1.0.0
 *
 * @package    Woocommerce_Simple_Storewide_Sale
 * @subpackage Woocommerce_Simple_Storewide_Sale/includes
 */

class Woocommerce_Simple_Storewide_Sale {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Wc_simple_storewide_sale_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Notices (array)
	 * @var array
	 */
	public $notices = array();

	/**
	 * The minimal version of woocommerce
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string
	 */
	protected $min_wc_version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name    = 'wc-simple-storewide-sale';
		$this->version        = '1.1.5';
		$this->min_wc_version = '2.4';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

		add_action( 'admin_notices', array( $this, 'admin_notices' ), 15 );

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Wc_simple_storewide_sale_Loader. Orchestrates the hooks of the plugin.
	 * - Wc_simple_storewide_sale_i18n. Defines internationalization functionality.
	 * - Wc_simple_storewide_sale_Admin. Defines all hooks for the admin area.
	 * - Wc_simple_storewide_sale_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wc-simple-storewide-sale-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wc-simple-storewide-sale-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wc-simple-storewide-sale-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-wc-simple-storewide-sale-public.php';

		if ( ! class_exists( 'Wpgenie_Dashboard' ) ) {
			require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-wpgenie-dashboard.php';

			$this->dashboard = new Wpgenie_Dashboard();
		}

		$this->loader = new Wc_simple_storewide_sale_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Wc_simple_storewide_sale_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Wc_simple_storewide_sale_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Wc_simple_storewide_sale_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'woocommerce_admin_field_datetimepicker', $plugin_admin, 'datetimepicker' );
		$this->loader->add_action( 'woocommerce_admin_field_ajaxproduct', $plugin_admin, 'ajaxproduct' );
		$this->loader->add_action( 'woocommerce_admin_field_hiddenhash', $plugin_admin, 'hiddenhash' );

		$this->loader->add_filter( 'plugin_row_meta', $plugin_admin, 'add_support_link', 10, 2 );
		$this->loader->add_filter( 'woocommerce_get_settings_pages', $plugin_admin, 'settings_class', 20 );

		$this->loader->add_filter( 'woocommerce_update_options_simple_storewide_sale', $this, 'set_onsale_page_transient', 90 );
		$this->loader->add_filter( 'woocommerce_delete_product_transients', $this, 'delete_onsale_page_transient', 90 );

		if ( ! version_compare( WC_VERSION, '2.7', '<' ) ) {
				$this->loader->add_filter( 'woocommerce_admin_settings_sanitize_option', $plugin_admin, 'sanitize_ajaxproduct_option', 20, 3 );
		}

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Wc_simple_storewide_sale_Public( $this->get_plugin_name(), $this->get_version(), $this->get_onsale_page_transient() );

		$this->loader->add_action( 'wp_footer', $plugin_public, 'sale_notice' );

		if ( $plugin_public->run_sales() && ! is_admin() ) {

			if ( version_compare( WC_VERSION, '2.7', '<' ) ) {
				$this->loader->add_filter( 'woocommerce_get_sale_price', $plugin_public, 'filterprice', 15, 2 );
				$this->loader->add_filter( 'woocommerce_get_price', $plugin_public, 'filterprice', 15, 2 );
			} else {
				$this->loader->add_filter( 'woocommerce_product_get_price', $plugin_public, 'filterprice', 15, 2 );
				$this->loader->add_filter( 'woocommerce_product_get_sale_price', $plugin_public, 'filterprice', 15, 2 );
				$this->loader->add_filter( 'woocommerce_product_variation_get_price', $plugin_public, 'filterprice', 15, 2 );
				$this->loader->add_filter( 'woocommerce_product_variation_get_sale_price', $plugin_public, 'filterprice', 15, 2 );
			}

			$this->loader->add_filter( 'woocommerce_variation_prices_price', $plugin_public, 'filterprice', 15, 3 );
			$this->loader->add_filter( 'woocommerce_variation_prices_sale_price', $plugin_public, 'filterprice', 10, 3 );
			$this->loader->add_filter( 'wc_onsale_page_product_ids_on_sale', $plugin_public, 'add_wc_onsale_page_product_ids_on_sale', 10, 1 );
			$this->loader->add_filter( 'woocommerce_get_variation_prices_hash', $plugin_public, 'add_discount_woocommerce_get_variation_prices_hash', 15, 2 );

		}

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$environment_warning = self::get_environment_warning();

		if ( $environment_warning ) {
			$this->add_admin_notice( 'bad_environment', 'error', $environment_warning );
		} else {
			$this->loader->run();
		}
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Wc_simple_storewide_sale_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

	function get_environment_warning() {
		if ( version_compare( WC_VERSION, $this->min_wc_version, '<' ) ) {
			$message = __( 'Woocommerce Simple Storewide Sale - The minimum WooCommerce version required for this plugin is %1$s. You are running %2$s.', 'wc-sbd', 'wc-sbd' );

			return sprintf( $message, $min_wc_version, WC_VERSION );
		}

		return false;
	}
	/**
	 * Allow this class and other classes to add slug keyed notices (to avoid duplication)
	 */
	public function add_admin_notice( $slug, $class, $message ) {
		$this->notices[ $slug ] = array(
			'class'   => $class,
			'message' => $message,
		);
	}

	/**
	 * Display any notices we've collected thus far (e.g. for connection, disconnection)
	 */
	public function admin_notices() {
		foreach ( (array) $this->notices as $notice_key => $notice ) {
			echo "<div class='" . esc_attr( $notice['class'] ) . "'><p>";
			echo wp_kses( $notice['message'], array( 'a' => array( 'href' => array() ) ) );
			echo '</p></div>';
		}
	}

	function set_onsale_page_transient() {

		$tax_query = array();
		$metaquery = array();

		$excludetype        = get_option( 'wcsss_exclude_type', '' );
		$excludecat         = get_option( 'wcsss_exclude_cat', '' );
		$includecat         = get_option( 'wcsss_include_cat', '' );
		$excludetags        = get_option( 'wcsss_exclude_tag', '' );
		$includetags        = get_option( 'wcsss_include_tag', '' );
		$excludeproduct_tmp = get_option( 'wcsss_exclude_product', '' );
		$includeproduct_tmp = get_option( 'wcsss_include_product', '' );
		$includesku_tmp     = get_option( 'wcsss_include_sku', '' );
		$excludesku_tmp     = get_option( 'wcsss_exclude_sku', '' );

		if ( ! empty( $excludetype ) ) {
			$tax_query[] = array(
				'taxonomy' => 'product_type',
				'field'    => 'slug',
				'terms'    => $excludetype, // Don't display products in the clothing category on the shop page.
				'operator' => 'NOT IN',
			);
		}

		if ( ! empty( $excludecat ) ) {
			$tax_query[] = array(
				'taxonomy' => 'product_cat',
				'field'    => 'id',
				'terms'    => $excludecat, // Don't display products in the clothing category on the shop page.
				'operator' => 'NOT IN',
			);
		}

		if ( ! empty( $includecat ) ) {
			$tax_query[] = array(
				'taxonomy' => 'product_cat',
				'field'    => 'id',
				'terms'    => $includecat, // Don't display products in the clothing category on the shop page.
				'operator' => 'IN',
			);
		}

		if ( ! empty( $excludetags ) ) {
			$tax_query[] = array(
				'taxonomy' => 'product_tag',
				'field'    => 'id',
				'terms'    => $excludetags, // Don't display products in the clothing category on the shop page.
				'operator' => 'NOT IN',
			);
		}
		if ( ! empty( $includetags ) ) {
			$tax_query[] = array(
				'taxonomy' => 'product_tag',
				'field'    => 'id',
				'terms'    => $includetags, // Don't display products in the clothing category on the shop page.
				'operator' => 'IN',
			);
		}
		if ( ! empty( $includesku_tmp ) ) {
			$metaquery[] =
				array(
					'key'     => '_sku',
					'value'   => explode( ',', $includesku_tmp ),
					'compare' => 'IN',
				);
		}
		if ( ! empty( $excludesku_tmp ) ) {
			$metaquery[] = array(
				'key'     => '_sku',
				'value'   => explode( ',', $excludesku_tmp ),
				'compare' => 'NOT IN',
			);
		}

		if ( ! empty( $excludeproduct_tmp ) ) {
			$query_args['post__not_in'] = $excludeproduct_tmp;
		}
		if ( ! empty( $includeproduct_tmp ) ) {
			$query_args['post__in'] = $includeproduct_tmp;
		}
		if ( ! empty( $tax_query ) ) {
			$query_args['tax_query'] = $tax_query;
		}
		if ( ! empty( $metaquery ) ) {
			$query_args['meta_query'] = $metaquery;
		}
			$query_args['post_type']      = 'product';
			$query_args['posts_per_page'] = '-1';

		$query = new WP_Query( $query_args );

		$product_ids_on_sale = wp_parse_id_list( array_merge( wp_list_pluck( $query->posts, 'ID' ), array_diff( wp_list_pluck( $query->posts, 'post_parent' ), array( 0 ) ) ) );

		set_transient( 'wc_onsale_page_products_onsale', $product_ids_on_sale, DAY_IN_SECONDS * 30 );

		return $product_ids_on_sale;
	}

	function delete_onsale_page_transient() {
		delete_transient( 'wc_onsale_page_products_onsale' );
		$this->set_onsale_page_transient();
	}

	function get_onsale_page_transient() {
		$product_ids_on_sale = get_transient( 'wc_onsale_page_products_onsale' );
		if ( false !== $product_ids_on_sale ) {
			return $product_ids_on_sale;
		} else {
			$product_ids_on_sale = $this->set_onsale_page_transient();
			return $product_ids_on_sale;
		}
	}


}
